@@ -144,6 +144,8 @@ static gchar     *terminal_screen_zoom_font                     (TerminalScreen
                                                                  TerminalZoomLevel      zoom);
 static void       terminal_screen_urgent_bell                   (TerminalWidget        *widget,
                                                                  TerminalScreen        *screen);
+static void       terminal_screen_set_custom_command            (TerminalScreen        *screen,
+                                                                 gchar                **command);
 
 
 
@@ -773,7 +775,7 @@ static gchar**
 terminal_screen_get_child_environment (TerminalScreen *screen)
 {
   GtkWidget     *toplevel;
-  gchar         *display_name;
+  const gchar   *display_name;
   gchar        **result;
   gchar        **p;
   guint          n;
@@ -816,9 +818,8 @@ terminal_screen_get_child_environment (TerminalScreen *screen)
       result[n++] = g_strdup_printf ("WINDOWID=%ld", (glong) gdk_x11_window_get_xid (gtk_widget_get_window (toplevel)));
 
       /* determine the DISPLAY value for the command */
-      display_name = gdk_screen_make_display_name (gtk_widget_get_screen (toplevel));
+      display_name = gdk_display_get_name (gdk_screen_get_display (gtk_widget_get_screen (toplevel)));
       result[n++] = g_strdup_printf ("DISPLAY=%s", display_name);
-      g_free (display_name);
     }
 #endif
 
@@ -1241,7 +1242,7 @@ terminal_screen_vte_selection_changed (VteTerminal    *terminal,
   /* copy vte selection to GDK_SELECTION_CLIPBOARD if option is set */
   g_object_get (G_OBJECT (screen->preferences),
                 "misc-copy-on-select", &copy_on_select, NULL);
-  if (copy_on_select)
+  if (copy_on_select && vte_terminal_get_has_selection (terminal))
     vte_terminal_copy_clipboard (terminal);
 
   g_signal_emit (G_OBJECT (screen), screen_signals[SELECTION_CHANGED], 0);
@@ -1520,6 +1521,58 @@ terminal_screen_urgent_bell (TerminalWidget *widget,
 
 
 
+static void
+terminal_screen_set_custom_command (TerminalScreen *screen,
+                                    gchar         **command)
+{
+  terminal_return_if_fail (TERMINAL_IS_SCREEN (screen));
+
+  if (G_UNLIKELY (screen->custom_command != NULL))
+    g_strfreev (screen->custom_command);
+
+  if (G_LIKELY (command != NULL && *command != NULL))
+    screen->custom_command = g_strdupv (command);
+  else
+    screen->custom_command = NULL;
+}
+
+
+
+/**
+ * terminal_screen_new:
+ * @command   : Command.
+ * @directory : Working directory.
+ * @title     : Terminal title.
+ * @hold      : Whether to hold the terminal after child exits.
+ * @columns   : Columns (width).
+ * @rows      : Rows (height).
+ *
+ * Creates a terminal screen object.
+ **/
+TerminalScreen *
+terminal_screen_new (gchar    **command,
+                     gchar     *directory,
+                     gchar     *title,
+                     gboolean   hold,
+                     glong      columns,
+                     glong      rows)
+{
+  TerminalScreen *screen = g_object_new (TERMINAL_TYPE_SCREEN, NULL);
+
+  if (command != NULL)
+    terminal_screen_set_custom_command (screen, command);
+  if (directory != NULL)
+    terminal_screen_set_working_directory (screen, directory);
+  if (title != NULL)
+    terminal_screen_set_custom_title (screen, title);
+  screen->hold = hold;
+  vte_terminal_set_size (VTE_TERMINAL (screen->terminal), columns, rows);
+
+  return screen;
+}
+
+
+
 /**
  * terminal_screen_launch_child:
  * @screen  : A #TerminalScreen.
@@ -1589,28 +1642,6 @@ terminal_screen_launch_child (TerminalScreen *screen)
 
 
 
-/**
- * terminal_screen_set_custom_command:
- * @screen  : A #TerminalScreen.
- * @command :
- **/
-void
-terminal_screen_set_custom_command (TerminalScreen *screen,
-                                    gchar         **command)
-{
-  terminal_return_if_fail (TERMINAL_IS_SCREEN (screen));
-
-  if (G_UNLIKELY (screen->custom_command != NULL))
-    g_strfreev (screen->custom_command);
-
-  if (G_LIKELY (command != NULL && *command != NULL))
-    screen->custom_command = g_strdupv (command);
-  else
-    screen->custom_command = NULL;
-}
-
-
-
 /**
  * terminal_screen_get_custom_title:
  * @screen  : A #TerminalScreen.
@@ -1990,24 +2021,6 @@ terminal_screen_set_working_directory (TerminalScreen *screen,
 
 
 
-/**
- * terminal_screen_set_hold:
- * @screen : A #TerminalScreen.
- * @hold   : %TRUE to keep @screen around when the child exits.
- *
- * Sets  whether the terminal screen will be destroyed when
- * the child exits.
- **/
-void
-terminal_screen_set_hold (TerminalScreen *screen,
-                          gboolean        hold)
-{
-  terminal_return_if_fail (TERMINAL_IS_SCREEN (screen));
-  screen->hold = hold;
-}
-
-
-
 /**
  * terminal_screen_has_selection:
  * @screen      : A #TerminalScreen.
@@ -2179,7 +2192,6 @@ terminal_screen_get_tab_label (TerminalScreen *screen)
 
   screen->tab_label = gtk_label_new (NULL);
   gtk_widget_set_margin_start (screen->tab_label, 2);
-  gtk_label_set_width_chars (GTK_LABEL (screen->tab_label), 10);
   gtk_box_pack_start  (GTK_BOX (hbox), screen->tab_label, TRUE, TRUE, 0);
   g_object_bind_property (G_OBJECT (screen), "title",
                           G_OBJECT (screen->tab_label), "label",
@@ -2354,17 +2366,20 @@ terminal_screen_update_scrolling_bar (TerminalScreen *screen)
 void
 terminal_screen_update_font (TerminalScreen *screen)
 {
+  GtkWidget            *toplevel = gtk_widget_get_toplevel (GTK_WIDGET (screen));
   gboolean              font_use_system, font_allow_bold;
   gchar                *font_name;
   PangoFontDescription *font_desc;
   glong                 grid_w = 0, grid_h = 0;
-  GtkWidget            *toplevel;
   GSettings            *settings;
 
   terminal_return_if_fail (TERMINAL_IS_SCREEN (screen));
   terminal_return_if_fail (TERMINAL_IS_PREFERENCES (screen->preferences));
   terminal_return_if_fail (VTE_IS_TERMINAL (screen->terminal));
 
+  if (!TERMINAL_IS_WINDOW (toplevel))
+    return;
+
   g_object_get (G_OBJECT (screen->preferences),
                 "font-use-system", &font_use_system,
                 "font-allow-bold", &font_allow_bold,
@@ -2379,18 +2394,15 @@ terminal_screen_update_font (TerminalScreen *screen)
   else
     g_object_get (G_OBJECT (screen->preferences), "font-name", &font_name, NULL);
 
-  toplevel = gtk_widget_get_toplevel (GTK_WIDGET (screen));
-  if (TERMINAL_IS_WINDOW (toplevel))
+  if (TERMINAL_WINDOW (toplevel)->font)
     {
-      if (TERMINAL_WINDOW (toplevel)->font)
-        {
-          g_free (font_name);
-          font_name = g_strdup (TERMINAL_WINDOW (toplevel)->font);
-        }
-      if (TERMINAL_WINDOW (toplevel)->zoom != TERMINAL_ZOOM_LEVEL_DEFAULT)
-        font_name = terminal_screen_zoom_font (screen, font_name, TERMINAL_WINDOW (toplevel)->zoom);
+      g_free (font_name);
+      font_name = g_strdup (TERMINAL_WINDOW (toplevel)->font);
     }
 
+  if (TERMINAL_WINDOW (toplevel)->zoom != TERMINAL_ZOOM_LEVEL_DEFAULT)
+    font_name = terminal_screen_zoom_font (screen, font_name, TERMINAL_WINDOW (toplevel)->zoom);
+
   if (gtk_widget_get_realized (GTK_WIDGET (screen)))
     terminal_screen_get_size (screen, &grid_w, &grid_h);
 
@@ -2403,8 +2415,8 @@ terminal_screen_update_font (TerminalScreen *screen)
       g_free (font_name);
     }
 
-  /* update window geometry it required */
-  if (grid_w > 0 && grid_h > 0)
+  /* update window geometry it required (not needed for drop-down) */
+  if (!TERMINAL_WINDOW (toplevel)->drop_down && grid_w > 0 && grid_h > 0)
     terminal_screen_force_resize_window (screen, GTK_WINDOW (toplevel), grid_w, grid_h);
 }
 
